from casadi import *

# Formulate the ODE
x = SX.sym('x', 2)
u = SX.sym('u')

xdot = vertcat(((1-x[1]**2)*x[0]-x[1]+u, x[0]))

f = SXFunction([x,u],[xdot])
f.init()


# End time
T = 10.

# Number of shooting nodes
N = 20
h = T/N

# Evaluate at some value
x_test = DMatrix([0.1,0.2])
u_test = 3

f.setInput(x_test,0)
# f.setInput(u_test,1)
f.evaluate()
xdot_test = f.getOutput(0)

[xdot_test] = f([x_test, u_test])
xdot_test = f([x_test, u_test])[0]
print xdot_test

# Discrete time dynamics
F = simpleRK(f, 10)
#F = simpleIRK(f)
#F = simpleIntegrator(f)

# Evaluate at some value
print F([x_test, u_test, h])

# Formulate the ODE
nw = 2*(N+1) + N
w = MX.sym("w",nw)
X = [w[3*k : 3*k+2] for k in range(N+1)]
U = [w[3*k+2] for k in range(N)]

# Objective function
J = inner_prod(w,w)/2

# Constraints
g = []
for k in range(N):
    [X_next] = F([X[k], U[k], h])
    g.append(X_next - X[k+1])
g = vertcat(g)

# Formulate the NLP
nlp = MXFunction( nlpIn(x=w), nlpOut(f=J, g=g) )
solver = NlpSolver("ipopt", nlp)
solver.init()

# Variable bounds and initial guess
w0 = -DMatrix.zeros(nw)
lbw = -DMatrix.inf(nw)
ubw = DMatrix.inf(nw)

# Control bounds
lbw[2::3] = -1
ubw[2::3] =  1

# State bounds
#lbw[0::3] = -0.25

# Initial conditions
lbw[0] = ubw[0] = 0
lbw[1] = ubw[1] = 1

# Solve the NLP
solver.setInput(w0,'x0')
solver.setInput(lbw,'lbx')
solver.setInput(ubw,'ubx')
solver.setInput(0,'lbg')
solver.setInput(0,'ubg')
solver.evaluate()
w_opt =  solver.getOutput('x')

# Get the solution
x0_opt = w_opt[0::3]
x1_opt = w_opt[1::3]
u_opt = w_opt[2::3]

# Visualize the solution
from matplotlib import pylab as plt
t = plt.linspace(0,T,N+1)
plt.step(t,vertcat((0,u_opt)))
plt.plot(t,x0_opt)
plt.plot(t,x1_opt)
plt.xlabel('t')
plt.legend(('u','x0','x1'))
plt.grid(True)
plt.show()
